#include <iostream>
#include <stdio.h>
#include <GL/glut.h>
#include <vector>
#include <math.h>
#include <fstream>
#include <string>
#include "Point.h"
#ifndef WIN32
#include <stdlib.h>
#include <string.h>
#endif

using namespace std;

vector<Point> pointlist;			//Vector holding all points read in from file.

//These global variables determine where the sliders are based on the user moving them around.  Also
//the scaling variables are initialized here.
float xnew_y_top = .2, xnew_y_bottom = -.2, rotate_angle_x = 0.0;
float ynew_y_top = .2, ynew_y_bottom = -.2, rotate_angle_y = 0.0;
float znew_y_top = .2, znew_y_bottom = -.2, rotate_angle_z = 0.0;
float scalenew_x_left = -.3, scalenew_x_right = .3, scale_amount = 1.0;

//Global variables used in calculating the middle of the object being displayed.
float x_max, x_min, y_max, y_min, z_max, z_min, middle_x, middle_y, middle_z;

//Used to identify display lists created for the object, sliders, and quit button.
GLuint object, xslider, yslider, zslider, scaleslider, quitbutton;

//Holds the width, height, and depth of the object to be read in.
float width = 0;
float height = 0;
float depth = 0;

int overall_width = 800; 
int overall_height = 600;

void init();				
void display();
void createdisplaylist();
void read();
void findmaxmin();
void idle();
void reshape(int w, int h);
void mousecb(int x, int y);
void buttoncb(int button, int state, int x, int y);
void bitmap_output(GLfloat x, GLfloat y, char *string, void *font);

int main(int argc, char **argv)
{
	read();						//Read in the Object
	findmaxmin();				//Determine the middle of the object using the max and min in each direction.
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition (50, 100);
	glutInitWindowSize(overall_width, overall_height);
	glutCreateWindow("Final Project");
    init();
	glutDisplayFunc(display);
	glutMouseFunc(buttoncb);
	glutMotionFunc(mousecb);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);
	createdisplaylist();		//Generate the display lists for the object, slider, and quit button.
	glutMainLoop();
}

void createdisplaylist()
{
	object = glGenLists(1);			//Display list for object read in.
	xslider = glGenLists(1);		//Display list for xslider.
	yslider = glGenLists(1);		//Display list for yslider.
	zslider = glGenLists(1);		//Display list for zslider.
	scaleslider = glGenLists(1);	//Display list for scale slider
	quitbutton = glGenLists(1);		//Display list for quit button

	//Generate the display list for the volumetric object.
	glNewList(object, GL_COMPILE);
		glBegin(GL_POINTS);
		for(int i = 0; i < pointlist.size(); i++)
		{
			if(pointlist[i].intensity != 0)
			{
				glColor3f(pointlist[i].intensity/255, 0.0, 0.0);
				glVertex3f(pointlist[i].x, pointlist[i].y, pointlist[i].z);
			}
		}
		glEnd();
	glEndList();

	//Generate the display list for the xslider.
	glNewList(xslider, GL_COMPILE);
		glBegin(GL_POLYGON);
			glVertex3f(-4,-3.2,0);
			glVertex3f(-3,-3.2,0);
			glVertex3f(-3,3.2,0);
			glVertex3f(-4, 3.2, 0);
		glEnd();
	glEndList();
		
	//Generate the display list for the yslider.
	glNewList(yslider, GL_COMPILE);
		glBegin(GL_POLYGON);
			glVertex3f(-1,-3.2,0);
			glVertex3f(0,-3.2,0);
			glVertex3f(0,3.2,0);
			glVertex3f(-1, 3.2, 0);
		glEnd();
	glEndList();

	//Generate the display list for the zslider.
	glNewList(zslider, GL_COMPILE);
		glBegin(GL_POLYGON);
			glVertex3f(2,-3.2,0);
			glVertex3f(3,-3.2,0);
			glVertex3f(3,3.2,0);
			glVertex3f(2, 3.2, 0);
		glEnd();
	glEndList();

	//Generate the display list for the scale slider.
	glNewList(scaleslider, GL_COMPILE);
		glBegin(GL_POLYGON);
			glVertex3f(-4.3, 4, 0);
			glVertex3f(-4.3, 4.3, 0);
			glVertex3f(4.3, 4.3, 0);
			glVertex3f(4.3, 4, 0);
		glEnd();
	glEndList();

	//Generate the display list for the quit button.
	glNewList(quitbutton, GL_COMPILE);
		glBegin(GL_POLYGON);
			glVertex3f(-2, -4, -0.01);
			glVertex3f(-2, -3.5, -0.01);
			glVertex3f(2, -3.5, -0.01);
			glVertex3f(2, -4, -0.01);
		glEnd();
	glEndList();
}

//This function will open a .raw volumetric file and read in all intenisty values.  While reading
//in the intensity files the x,y,z position of each point will be calculated by its position in the file.
void read()
{
	char * name = (char*) malloc (sizeof(char) * 50);	//Holds name of file.

	//This next section sets up a small menu that can be used to either open the block file, monster file, or colon file.
	//It will also make the user reenter a value if the value is invalid.
	int MenuNumber;
	bool redo = true;
	cout << "Main Menu" << endl;
	cout << "1. Block" << endl;
	cout << "2. Monster" << endl;
	cout << "3. Colon" << endl;
	cout << endl;
	cout << "**Note** If the colon image is chosen it could take several minutes to display" << endl << "due to the generation of a display list." << endl;
	cout << endl;
	do{
		cout << "Please enter a number shown above corresponding to the image you would like to" << endl << "see: (1-3)" << endl;
		cin >> MenuNumber;
		switch (MenuNumber) {
			case 1:
				width = 34;
				height = 9;
				depth = 54;
				name = "block.raw";
				redo = true;
				break;
			case 2:
				width = 87;
				height = 39;
				depth = 62;
				name = "monster.raw";
				redo = true;
				break;
			case 3:
				width = 204;
				height = 132;
				depth = 260;
				name = "colon.raw";
				redo = true;
				break;
			default:
				cout << "Value entered was not valid please choose between 1-3 from the menu." << endl;
				redo = false;
				break;
		}
	}
	while(redo == false);

	int xcount = 1;			//Used to count in the x direction to calculate point position.
	int ycount = 1;			//Used to count in the y direction to calculate point position.
	int zcount = 1;			//Used to count in the z direction to calculate point position.
	FILE * file;
	file = fopen (name,"rb");
	unsigned char * intensity;
    intensity = (unsigned char*) malloc (sizeof(unsigned char)*1);
	if(file == NULL)
		cout << "file not there" << endl;
	while(fread(intensity, sizeof(unsigned char), 1, file))
	{
		Point point;
		point.x = xcount;
		xcount++;

		point.y = ycount;

		point.z = zcount;
		point.intensity = intensity[0];

		//If a row has been completed then reset xcount back to zero and increase y by one.
		if(xcount == width)
		{
			xcount = 0;
			ycount++;
		}

		//If all colums in this slice has been completed then reset ycount back to zero and increase z by one.
		if(ycount == height && xcount == 0)
		{
			xcount = 0;
			ycount = 0;
			zcount++;
		}
		pointlist.push_back(point);	//Put the point that was read in into the vector of points.
	}
}

//Initializes the projections for opengl.
void init()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);				//Clears background to white.
	glClearDepth(1.0);		
	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);						//Makes sure things in background can not be seen.

	glMatrixMode(GL_PROJECTION);

    glLoadIdentity();
	glOrtho(-150,150,-150,150,-150,150);					//Set up the ortho projection.
	glMatrixMode(GL_MODELVIEW);
}

//Actually display the object and all sliders.
void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();

	//Make sure to set the projection for this viewport before continuing.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-150,150,-150,150,-150,150);
	glMatrixMode(GL_MODELVIEW);


	//Scale, rotate, and translate the object in the left side viewport.  The scale, rotate, and translate,
	//will only happen if neccessary.
	glScalef(scale_amount,scale_amount,scale_amount);
	glRotatef(rotate_angle_x, 1, 0, 0);
	glRotatef(rotate_angle_y, 0, 1, 0);
	glRotatef(rotate_angle_z, 0, 0, 1);
	glTranslatef(-middle_x, -middle_y, -middle_z);
	glViewport(0,0,.75*overall_width,overall_height);
	glCallList(object);
	glPopMatrix();

	glPushMatrix();

	//Set up a different project for the interface in the right viewport.
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	glOrtho(-5,5,-5,5,-5,5);
	glMatrixMode(GL_MODELVIEW);

	glViewport(.75*overall_width,0,.25*overall_width,overall_height);
	//Draw the background for the interface viewport.
	glColor3f(1.0,1.0,1.0);
	glBegin(GL_POLYGON);
		glVertex3f(5,-5,-2);
		glVertex3f(-5,-5,-2);
		glVertex3f(-5,5,-2);
		glVertex3f(5,5,-2);
	glEnd();

	//Draw the rotate in x direction slider
	glColor3f(0.0,0.0,1.0);
	glCallList(xslider);				//Use xslider call list

	//Draw the actual slider
	glColor3f(1.0,0.0,0.0);
	glBegin(GL_POLYGON);
		glVertex3f(-4, xnew_y_bottom, .01);
		glVertex3f(-3, xnew_y_bottom, .01);
		glVertex3f(-3, xnew_y_top, .01);
		glVertex3f(-4, xnew_y_top, .01);
	glEnd();

	//Draw the rotate in y direction slider
	glColor3f(0.0,0.0,1.0);
	glCallList(yslider);

	//Draw the actual slider
	glColor3f(1.0,0.0,0.0);
	glBegin(GL_POLYGON);
		glVertex3f(-1, ynew_y_bottom, .01);
		glVertex3f(0, ynew_y_bottom, .01);
		glVertex3f(0, ynew_y_top, .01);
		glVertex3f(-1, ynew_y_top, .01);
	glEnd();

	//Draw the rotate in z direction slider
	glColor3f(0.0,0.0,1.0);
	glCallList(zslider);

	//Draw the actual slider
	glColor3f(1.0,0.0,0.0);
	glBegin(GL_POLYGON);
		glVertex3f(2, znew_y_bottom, .01);
		glVertex3f(3, znew_y_bottom, .01);
		glVertex3f(3, znew_y_top, .01);
		glVertex3f(2, znew_y_top, .01);
	glEnd();

	//Draw the Scaling slider
	glColor3f(0.0, 0.0, 1.0);
	glCallList(scaleslider);

	//Draw the actual slider
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_POLYGON);
		glVertex3f(scalenew_x_left, 4, .01);
		glVertex3f(scalenew_x_left, 4.3, .01);
		glVertex3f(scalenew_x_right, 4.3, .01);
		glVertex3f(scalenew_x_right, 4, .01);
	glEnd();
	glPopMatrix();

	//Draw the quit button
	glColor3f(1.0, 0.0, 0.0);
	glCallList(quitbutton);

	//Draw the scale label
	glColor3f(1.0,0.0,0.0);
	bitmap_output(-2.5, 4.5, "+  Scale  -", GLUT_BITMAP_TIMES_ROMAN_24);
	//Draw the x rotate label
	bitmap_output(-4.3, 3.4, "Rotate X", GLUT_BITMAP_TIMES_ROMAN_10);
	//Draw the y rotate label
	bitmap_output(-1.3, 3.4, "Rotate Y", GLUT_BITMAP_TIMES_ROMAN_10);
	//Draw the z rotate label
	bitmap_output( 1.7, 3.4, "Rotate Z", GLUT_BITMAP_TIMES_ROMAN_10);
	//Draw the word "quit" on the quit button.
	glColor3f(1.0,1.0,1.0);
	bitmap_output(-1.3, -3.85, "Quit", GLUT_BITMAP_TIMES_ROMAN_24);

	glutSwapBuffers();						//Swaps buffers due to double buffering.
	glFlush();
}

void mousecb(int x, int y)
{
	//Part for if scaling slider is being used.
	if(x >= 620 && x <= 780 && y >= 42 && y <= 60)
	{
		if(x > 700)
		{
			scalenew_x_left = (((float)x - 700.0)/20.0) - .3;
			scalenew_x_right = (((float)x - 700.0)/20.0) + .3;
			scale_amount = -(((float)x - 700)/80.0)+1.0;
			display();
		}
		if(x <= 700)
		{
			scalenew_x_left = (((float)x - 700.0)/20.0) - .3;
			scalenew_x_right = (((float)x - 700.0)/20.0) + .3;
			scale_amount = -(((float)x - 700)/20.0)+1.0;
			display();
		}
	}

	//Part for if x rotate slider is being used.
	if(x >= 620 && x <= 640 && y >= 120 && y <= 480)
	{
		if(y > 300)
		{
			xnew_y_bottom = -(((float)y - 300.0)/60.0) - .2;
			xnew_y_top = -(((float)y - 300.0)/60.0) + .2;
			rotate_angle_x = (((float)y - 300.0)/180.0)*180;
			display();
		}
		if(y <= 300)
		{
			xnew_y_bottom = -(((float)y - 300.0)/60.0) - .2;
			xnew_y_top = -(((float)y - 300.0)/60.0) + .2;
			rotate_angle_x = (((float)y - 300.0)/180.0)*180;
			display();
		}
	}

	//Part for if y rotate slider is being used.
	if(x >= 680 && x <= 698 && y >= 120 && y <= 480)
	{
		if(x > 300)
		{
			ynew_y_bottom = -(((float)y - 300.0)/60.0) - .2;
			ynew_y_top = -(((float)y - 300.0)/60.0) + .2;
			rotate_angle_y = (((float)y - 300.0)/180.0)*180;
			display();
		}
		if(x <= 300)
		{
			ynew_y_bottom = -(((float)y - 300.0)/60.0) - .2;
			ynew_y_top = -(((float)y - 300.0)/60.0) + .2;
			rotate_angle_y = (((float)y - 300.0)/180.0)*180;
			display();
		}
	}

	//Part for if z rotate slider is being used.
	if(x >= 740 && x <= 760 && y >= 120 && y <= 480)
	{
		if(x > 300)
		{
			znew_y_bottom = -(((float)y - 300.0)/60.0) - .2;
			znew_y_top = -(((float)y - 300.0)/60.0) + .2;
			rotate_angle_z = (((float)y - 300.0)/180.0)*180;
			display();
		}
		if(x <= 300)
		{
			znew_y_bottom = -(((float)y - 300.0)/60.0) - .2;
			znew_y_top = -(((float)y - 300.0)/60.0) + .2;
			rotate_angle_z = (((float)y - 300.0)/180.0)*180;
			display();
		}
	}
}

void idle()
{
	glutPostRedisplay();
}

//This function will find the max and min for x,y,z.  Then determine the middle of the volumetric object.
void findmaxmin()
{

	float largest_x = -999999;
	float smallest_x = 999999;
	float largest_y = -999999;
	float smallest_y = 999999;
	float largest_z = -999999;
	float smallest_z = 999999;

	//Go through the point list and determine the max and min in each direction.
	for(int i = 0; i < pointlist.size(); i++)
	{
		if(pointlist[i].x > largest_x)
		{
			largest_x = pointlist[i].x;
			x_max = pointlist[i].x;
		}
		if(pointlist[i].x < smallest_x)
		{
			smallest_x = pointlist[i].x;
			x_min = pointlist[i].x;
		}
		if(pointlist[i].y > largest_y)
		{
			largest_y = pointlist[i].y;
			y_max = pointlist[i].y;
		}
		if(pointlist[i].y < smallest_y)
		{
			smallest_y = pointlist[i].y;
			y_min = pointlist[i].y;
		}
		if(pointlist[i].z > largest_z)
		{
			largest_z = pointlist[i].z;
			z_max = pointlist[i].z;
		}
		if(pointlist[i].z < smallest_z)
		{
			smallest_z = pointlist[i].z;
			z_min = pointlist[i].z;
		}
	}
	
	//Calculate the middle of each axis for the object.
	middle_x = (x_max + x_min)/2;
	middle_y = (y_max + y_min)/2;
	middle_z = (z_max + z_min)/2;

}

//Outputs a bitmap string to the screen.
void bitmap_output(GLfloat x, GLfloat y, char *string, void *font)
{
	int len, i;
	glRasterPos2f(x,y);
	len = (int) strlen(string);
	for (i = 0; i < len; i++)
		glutBitmapCharacter(font, string[i]);
}

void buttoncb(int button, int state, int x, int y)
{
	//If user clicks the reset button, return all values back to actual time setting.
	if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN && x >= 660 && x <= 740 && y >= 510 && y <= 540)
	{
		exit(0);
	}
}

void reshape(int w, int h)
{
	overall_width = w;
	overall_height = h;
}
